==> drm-1.0.tar.gz: drm-1.0/common/DrmSimulation.cpp <==
/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2001
 *
 * Author(s):
 *	Volker Fischer
 *
 * Description:
 *	DRM-simulation
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#include "DrmSimulation.h"


/* Implementation *************************************************************/
void CDRMSimulation::Run()
{
/*
 The hand over of data is done via an intermediate-buffer. The calling
 convention is always "input-buffer, output-buffer". Additional, the
 DRM-parameters are fed to the function
*/

	/* Initialization of the modules */
	Init();

	/* Set run flag */
	Param.bRunThread = TRUE;

	while (Param.bRunThread)
	{
		/**********************************************************************\
		* Transmitter														   *
		\**********************************************************************/
		/* MSC -------------------------------------------------------------- */
		/* Read the source signal */
		GenSimData.ReadData(Param, DataBuf);

		/* MLC-encoder */
		MSCMLCEncoder.ProcessData(Param, DataBuf, MLCEncBuf);

		/* Convolutional interleaver */
		SymbInterleaver.ProcessData(Param, MLCEncBuf, IntlBuf);


		/* FAC -------------------------------------------------------------- */
		GenerateFACData.ReadData(Param, GenFACDataBuf);
		FACMLCEncoder.ProcessData(Param, GenFACDataBuf, FACMapBuf);


		/* SDC -------------------------------------------------------------- */
		GenerateSDCData.ReadData(Param, GenSDCDataBuf);
		SDCMLCEncoder.ProcessData(Param, GenSDCDataBuf, SDCMapBuf);


		/* Mapping of the MSC, FAC, SDC and pilots on the carriers */
		OFDMCellMapping.ProcessData(Param, IntlBuf, FACMapBuf,
			SDCMapBuf, CarMapBuf);

		/* OFDM-modulation */
		OFDMModulation.ProcessData(Param, CarMapBuf, OFDMModBuf);



		/**********************************************************************\
		* Channel    														   *
		\**********************************************************************/
		DRMChannel.TransferData(Param, OFDMModBuf, RecDataBuf);



		/**********************************************************************\
		* Receiver    														   *
		\**********************************************************************/
switch (Param.eSimType)
{
case CParameter::ST_MSECHANEST:
case CParameter::ST_BER_IDEALCHAN:
		/* MSE of channel estimation, ideal channel estimation -------------- */
		/* Special OFDM demodulation for channel estimation tests (with guard-
		   interval removal) */
		OFDMDemodSimulation.ProcessDataOut(Param, RecDataBuf,
			ChanEstInBufSim, OFDMDemodBufChan2);

		/* Channel estimation and equalization */
		ChannelEstimation.ProcessData(Param, ChanEstInBufSim, ChanEstBuf);

		/* Ideal channel estimation (with MSE calculation) */
		IdealChanEst.ProcessDataIn(Param, ChanEstBuf, OFDMDemodBufChan2,
			ChanEstBuf);
	break;


case CParameter::ST_BITERROR:
		/* Bit error rate (we can use all synchronization units here!) ------ */
		/* This module converts the "CChanSimDataMod" data type of "DRMChannel"
		   to the "_REAL" data type, because a regular module can only have ONE
		   type of input buffers */
		DataConvChanResam.ProcessData(Param, RecDataBuf, ChanResInBuf);

		/* Frequency synchronization acquisition */
		FreqSyncAcq.ProcessData(Param, ChanResInBuf, FreqSyncAcqBuf);

		/* Resample input DRM-stream */
		InputResample.ProcessData(Param, FreqSyncAcqBuf, InpResBuf);

		/* Time synchronization */
		TimeSync.ProcessData(Param, InpResBuf, TimeSyncBuf);

		/* OFDM-demodulation */
		OFDMDemodulation.ProcessData(Param, TimeSyncBuf, OFDMDemodBuf);

		/* Synchronization in the frequency domain (using pilots) */
		SyncUsingPil.ProcessData(Param, OFDMDemodBuf, SyncUsingPilBuf);

		/* Channel estimation and equalization */
		ChannelEstimation.ProcessData(Param, SyncUsingPilBuf, ChanEstBuf);
	break;
}

		/* Demapping of the MSC, FAC, SDC and pilots from the carriers */
		OFDMCellDemapping.ProcessData(Param, ChanEstBuf,
			MSCCarDemapBuf, FACCarDemapBuf, SDCCarDemapBuf);

		/* FAC */
		FACMLCDecoder.ProcessData(Param, FACCarDemapBuf, FACDecBuf);
		UtilizeFACData.WriteData(Param, FACDecBuf);


		/* SDC */
		SDCMLCDecoder.ProcessData(Param, SDCCarDemapBuf, SDCDecBuf);
		UtilizeSDCData.WriteData(Param, SDCDecBuf);


		/* MSC */
		/* Symbol de-interleaver */
		SymbDeinterleaver.ProcessData(Param, MSCCarDemapBuf, DeintlBuf);

		/* MLC-decoder */
		MSCMLCDecoder.ProcessData(Param, DeintlBuf, MSCMLCDecBuf);

		/* Evaluate simulation data */
		EvaSimData.WriteData(Param, MSCMLCDecBuf);
	}
}

void CDRMSimulation::Init()
{
	/* Defines number of cells, important! */
	OFDMCellMapping.Init(Param, CarMapBuf);

	/* Defines number of SDC bits per super-frame */
	SDCMLCEncoder.Init(Param, SDCMapBuf);
	
	MSCMLCEncoder.Init(Param, MLCEncBuf);
	SymbInterleaver.Init(Param, IntlBuf);
	GenerateFACData.Init(Param, GenFACDataBuf);
	FACMLCEncoder.Init(Param, FACMapBuf);
	GenerateSDCData.Init(Param, GenSDCDataBuf);
	OFDMModulation.Init(Param, OFDMModBuf);
	GenSimData.Init(Param, DataBuf);


	/* Receiver modules */
	/* The order of modules are important! */
	InputResample.Init(Param, InpResBuf);
	FreqSyncAcq.Init(Param, FreqSyncAcqBuf);
	TimeSync.Init(Param, TimeSyncBuf);
	SyncUsingPil.Init(Param, SyncUsingPilBuf);

	/* Channel estimation init must be called before OFDMDemodSimulation
	   module, because the delay is set here which the other modules use! */
	ChannelEstimation.Init(Param, ChanEstBuf);
	
	OFDMCellDemapping.Init(Param, MSCCarDemapBuf, FACCarDemapBuf, 
		SDCCarDemapBuf);
	FACMLCDecoder.Init(Param, FACDecBuf);
	UtilizeFACData.Init(Param);
	SDCMLCDecoder.Init(Param, SDCDecBuf);
	UtilizeSDCData.Init(Param);
	SymbDeinterleaver.Init(Param, DeintlBuf);
	MSCMLCDecoder.Init(Param, MSCMLCDecBuf);


	/* Special module for simulation */
	EvaSimData.Init(Param);

	/* Init channel. The channel must be initialized before the modules
	   "OFDMDemodSimulation" and "IdealChanEst" because they need iNumTaps and
	   tap delays in global struct */
	DRMChannel.Init(Param, RecDataBuf);

	/* Mode dependent initializations */
	switch (Param.eSimType)
	{
	case CParameter::ST_MSECHANEST:
	case CParameter::ST_BER_IDEALCHAN:
		/* Init OFDM demod before IdealChanEst, because the timing offset of
		   useful part extraction is set here */
		OFDMDemodSimulation.Init(Param, ChanEstInBufSim, OFDMDemodBufChan2);

		/* Problem: "ChanEstBuf" is used for input and output buffer. That only
		   works with single buffers. This solution works for this case but is
		   not a very nice solution FIXME */
		IdealChanEst.Init(Param, ChanEstBuf);
		break;

	case CParameter::ST_BITERROR:
		DataConvChanResam.Init(Param, ChanResInBuf);

		OFDMDemodulation.Init(Param, OFDMDemodBuf);
		break;
	}

	/* We only want to simulate tracking performance ------------------------ */
	TimeSync.StopTimingAcqu();
	TimeSync.StopRMDetAcqu(); /* Robustness mode detection */
	ChannelEstimation.GetTimeSyncTrack()->StartTracking();

	/* We stop tracking of time wiener interpolation since during acquisition,
	   no automatic update of the statistic estimates is done. We need that
	   because we set the correct parameters once in the init routine */
	ChannelEstimation.GetTimeWiener()->StopTracking();

	/* Disable FAC evaluation to make sure that no mistakenly correct CRC
	   sets false parameters which can cause run-time errors */
	UtilizeFACData.SetSyncInput(TRUE);

	/* We have to first start aquisition and then stop it right after it to set
	   internal parameters */
	SyncUsingPil.StartAcquisition();
	SyncUsingPil.StopAcquisition();

	SyncUsingPil.StartTrackPil();
}

CDRMSimulation::CDRMSimulation()
{
	/* Set all parameters to meaningful value for startup state. If we want to
	   make a simulation we just have to specify the important values */
	/* Init streams */
	Param.ResetServicesStreams();

	/* Use 6.3.6 to set this two parameters! */
	Param.FACRepetition[0] = 0;
	Param.FACNumRep = 1;

	/* Date, time */
	Param.iDay = 0;
	Param.iMonth = 0;
	Param.iYear = 0;
	Param.iUTCHour = 0;
	Param.iUTCMin = 0;

	/* Frame IDs */
	Param.iFrameIDTransm = 0;
	Param.iFrameIDReceiv = 0;

	/* Initialize synchronization parameters */
	Param.rResampleOffset = (_REAL) 0.0;
	Param.rFreqOffsetAcqui = (_REAL) 0.0;
	Param.rFreqOffsetTrack = (_REAL) 0.0;
	Param.iTimingOffsTrack = 0;

	Param.InitCellMapTable(RM_ROBUSTNESS_MODE_B, SO_3);
	Param.iNumAudioService = 1;
	Param.iNumDataService = 0;
	Param.Service[0].AudioParam.iStreamID = 0;

	Param.MSCPrLe.iPartA = 1;
	Param.MSCPrLe.iPartB = 1;
	Param.MSCPrLe.iHierarch = 0;

	Param.Stream[0].iLenPartA = 0; // EEP, if "= 0"
	Param.eSymbolInterlMode = CParameter::SI_SHORT;
	Param.eMSCCodingScheme = CParameter::CS_3_SM;
	Param.eSDCCodingScheme = CParameter::CS_2_SM;

	/* DRM channel parameters */
	Param.iDRMChannelNo = 1;
	Param.rSimSNRdB = 25;
}

==> drm-1.1.tar.gz: drm-1.1/common/DrmSimulation.cpp <==
/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2001
 *
 * Author(s):
 *	Volker Fischer
 *
 * Description:
 *	DRM-simulation
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#include "DrmSimulation.h"


/* Implementation *************************************************************/
void CDRMSimulation::Run()
{
/*
 The hand over of data is done via an intermediate-buffer. The calling
 convention is always "input-buffer, output-buffer". Additional, the
 DRM-parameters are fed to the function
*/

	/* Initialization of the modules */
	Init();

	/* Set run flag */
	Param.bRunThread = TRUE;

	while (Param.bRunThread)
	{
		/**********************************************************************\
		* Transmitter														   *
		\**********************************************************************/
		/* MSC -------------------------------------------------------------- */
		/* Read the source signal */
		GenSimData.ReadData(Param, DataBuf);

		/* MLC-encoder */
		MSCMLCEncoder.ProcessData(Param, DataBuf, MLCEncBuf);

		/* Convolutional interleaver */
		SymbInterleaver.ProcessData(Param, MLCEncBuf, IntlBuf);


		/* FAC -------------------------------------------------------------- */
		GenerateFACData.ReadData(Param, GenFACDataBuf);
		FACMLCEncoder.ProcessData(Param, GenFACDataBuf, FACMapBuf);


		/* SDC -------------------------------------------------------------- */
		GenerateSDCData.ReadData(Param, GenSDCDataBuf);
		SDCMLCEncoder.ProcessData(Param, GenSDCDataBuf, SDCMapBuf);


		/* Mapping of the MSC, FAC, SDC and pilots on the carriers */
		OFDMCellMapping.ProcessData(Param, IntlBuf, FACMapBuf,
			SDCMapBuf, CarMapBuf);

		/* OFDM-modulation */
		OFDMModulation.ProcessData(Param, CarMapBuf, OFDMModBuf);



		/**********************************************************************\
		* Channel    														   *
		\**********************************************************************/
		DRMChannel.TransferData(Param, OFDMModBuf, RecDataBuf);



		/**********************************************************************\
		* Receiver    														   *
		\**********************************************************************/
switch (Param.eSimType)
{
case CParameter::ST_MSECHANEST:
case CParameter::ST_BER_IDEALCHAN:
		/* MSE of channel estimation, ideal channel estimation -------------- */
		/* Special OFDM demodulation for channel estimation tests (with guard-
		   interval removal) */
		OFDMDemodSimulation.ProcessDataOut(Param, RecDataBuf,
			ChanEstInBufSim, OFDMDemodBufChan2);

		/* Channel estimation and equalization */
		ChannelEstimation.ProcessData(Param, ChanEstInBufSim, ChanEstBuf);

		/* Ideal channel estimation (with MSE calculation) */
		IdealChanEst.ProcessDataIn(Param, ChanEstBuf, OFDMDemodBufChan2,
			ChanEstBuf);
	break;


default: /* Other types like ST_BITERROR or ST_SYNC_PARAM */
		/* Bit error rate (we can use all synchronization units here!) ------ */
		/* This module converts the "CChanSimDataMod" data type of "DRMChannel"
		   to the "_REAL" data type, because a regular module can only have ONE
		   type of input buffers */
		DataConvChanResam.ProcessData(Param, RecDataBuf, ChanResInBuf);

		/* Frequency synchronization acquisition */
		FreqSyncAcq.ProcessData(Param, ChanResInBuf, FreqSyncAcqBuf);

		/* Resample input DRM-stream */
		InputResample.ProcessData(Param, FreqSyncAcqBuf, InpResBuf);

		/* Time synchronization */
		TimeSync.ProcessData(Param, InpResBuf, TimeSyncBuf);

		/* OFDM-demodulation */
		OFDMDemodulation.ProcessData(Param, TimeSyncBuf, OFDMDemodBuf);

		/* Synchronization in the frequency domain (using pilots) */
		SyncUsingPil.ProcessData(Param, OFDMDemodBuf, SyncUsingPilBuf);

		/* Channel estimation and equalization */
		ChannelEstimation.ProcessData(Param, SyncUsingPilBuf, ChanEstBuf);
	break;
}

		/* Demapping of the MSC, FAC, SDC and pilots from the carriers */
		OFDMCellDemapping.ProcessData(Param, ChanEstBuf,
			MSCCarDemapBuf, FACCarDemapBuf, SDCCarDemapBuf);

		/* FAC */
		FACMLCDecoder.ProcessData(Param, FACCarDemapBuf, FACDecBuf);
		UtilizeFACData.WriteData(Param, FACDecBuf);


		/* SDC */
		SDCMLCDecoder.ProcessData(Param, SDCCarDemapBuf, SDCDecBuf);
		UtilizeSDCData.WriteData(Param, SDCDecBuf);


		/* MSC */
		/* Symbol de-interleaver */
		SymbDeinterleaver.ProcessData(Param, MSCCarDemapBuf, DeintlBuf);

		/* MLC-decoder */
		MSCMLCDecoder.ProcessData(Param, DeintlBuf, MSCMLCDecBuf);

		/* Evaluate simulation data */
		EvaSimData.WriteData(Param, MSCMLCDecBuf);
	}
}

void CDRMSimulation::Init()
{
	/* Defines number of cells, important! */
	OFDMCellMapping.Init(Param, CarMapBuf);

	/* Defines number of SDC bits per super-frame */
	SDCMLCEncoder.Init(Param, SDCMapBuf);
	
	MSCMLCEncoder.Init(Param, MLCEncBuf);
	SymbInterleaver.Init(Param, IntlBuf);
	GenerateFACData.Init(Param, GenFACDataBuf);
	FACMLCEncoder.Init(Param, FACMapBuf);
	GenerateSDCData.Init(Param, GenSDCDataBuf);
	OFDMModulation.Init(Param, OFDMModBuf);
	GenSimData.Init(Param, DataBuf);


	/* Receiver modules */
	/* The order of modules are important! */
	InputResample.Init(Param, InpResBuf);
	FreqSyncAcq.Init(Param, FreqSyncAcqBuf);
	TimeSync.Init(Param, TimeSyncBuf);
	SyncUsingPil.Init(Param, SyncUsingPilBuf);

	/* Channel estimation init must be called before OFDMDemodSimulation
	   module, because the delay is set here which the other modules use! */
	ChannelEstimation.Init(Param, ChanEstBuf);
	
	OFDMCellDemapping.Init(Param, MSCCarDemapBuf, FACCarDemapBuf, 
		SDCCarDemapBuf);
	FACMLCDecoder.Init(Param, FACDecBuf);
	UtilizeFACData.Init(Param);
	SDCMLCDecoder.Init(Param, SDCDecBuf);
	UtilizeSDCData.Init(Param);
	SymbDeinterleaver.Init(Param, DeintlBuf);
	MSCMLCDecoder.Init(Param, MSCMLCDecBuf);


	/* Special module for simulation */
	EvaSimData.Init(Param);

	/* Init channel. The channel must be initialized before the modules
	   "OFDMDemodSimulation" and "IdealChanEst" because they need iNumTaps and
	   tap delays in global struct */
	DRMChannel.Init(Param, RecDataBuf);

	/* Mode dependent initializations */
	switch (Param.eSimType)
	{
	case CParameter::ST_MSECHANEST:
	case CParameter::ST_BER_IDEALCHAN:
		/* Init OFDM demod before IdealChanEst, because the timing offset of
		   useful part extraction is set here */
		OFDMDemodSimulation.Init(Param, ChanEstInBufSim, OFDMDemodBufChan2);

		/* Problem: "ChanEstBuf" is used for input and output buffer. That only
		   works with single buffers. This solution works for this case but is
		   not a very nice solution FIXME */
		IdealChanEst.Init(Param, ChanEstBuf);
		break;

	default: /* Other types like ST_BITERROR or ST_SYNC_PARAM */
		DataConvChanResam.Init(Param, ChanResInBuf);

		OFDMDemodulation.Init(Param, OFDMDemodBuf);

		/* Set correct position for Hilbert filter */
		TimeSync.SetFilterTaps(Param.rFreqOffsetAcqui);
		break;
	}

	/* We only want to simulate tracking performance ------------------------ */
	TimeSync.StopTimingAcqu();
	TimeSync.StopRMDetAcqu(); /* Robustness mode detection */
	ChannelEstimation.GetTimeSyncTrack()->StartTracking();

	/* We stop tracking of time wiener interpolation since during acquisition,
	   no automatic update of the statistic estimates is done. We need that
	   because we set the correct parameters once in the init routine */
	ChannelEstimation.GetTimeWiener()->StopTracking();

	/* Disable FAC evaluation to make sure that no mistakenly correct CRC
	   sets false parameters which can cause run-time errors */
	UtilizeFACData.SetSyncInput(TRUE);

	/* We have to first start aquisition and then stop it right after it to set
	   internal parameters */
	SyncUsingPil.StartAcquisition();
	SyncUsingPil.StopAcquisition();

	SyncUsingPil.StartTrackPil();
}

CDRMSimulation::CDRMSimulation() : iSimTime(0), iSimNumErrors(0),
	rStartSNR((_REAL) 0.0), rEndSNR((_REAL) 0.0), rStepSNR((_REAL) 0.0)
{
	/* Set all parameters to meaningful value for startup state. If we want to
	   make a simulation we just have to specify the important values */
	/* Init streams */
	Param.ResetServicesStreams();


	/* Service paramters (only use service 0) ------------------------------- */
	/* Data service */
	Param.iNumAudioService = 0;
	Param.iNumDataService = 1;

	Param.Service[0].eAudDataFlag = CParameter::SF_DATA;
	Param.Service[0].DataParam.iStreamID = 0;
	Param.Service[0].DataParam.ePacketModInd = CParameter::PM_SYNCHRON_STR_MODE;
	Param.Service[0].iServiceID = 1; /* Service ID must be set for activation */

	/* Stream */
	Param.Stream[0].iLenPartA = 0; // EEP, if "= 0"


	/* Date, time */
	Param.iDay = 0;
	Param.iMonth = 0;
	Param.iYear = 0;
	Param.iUTCHour = 0;
	Param.iUTCMin = 0;

	/* Frame IDs */
	Param.iFrameIDTransm = 0;
	Param.iFrameIDReceiv = 0;

	/* Initialize synchronization parameters */
	Param.rResampleOffset = (_REAL) 0.0;
	Param.rFreqOffsetAcqui = (_REAL) VIRTUAL_INTERMED_FREQ / SOUNDCRD_SAMPLE_RATE;
	Param.rFreqOffsetTrack = (_REAL) 0.0;
	Param.iTimingOffsTrack = 0;

	Param.InitCellMapTable(RM_ROBUSTNESS_MODE_B, SO_3);

	Param.MSCPrLe.iPartA = 1;
	Param.MSCPrLe.iPartB = 1;
	Param.MSCPrLe.iHierarch = 0;

	Param.eSymbolInterlMode = CParameter::SI_SHORT;
	Param.eMSCCodingScheme = CParameter::CS_3_SM;
	Param.eSDCCodingScheme = CParameter::CS_2_SM;

	/* DRM channel parameters */
	Param.iDRMChannelNum = 1;
	Param.rSimSNRdB = 25;
}
